;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC16F876A. This file contains the basic code     *
;   building blocks to build upon.                                    *  
;                                                                     *
;   If interrupts are not used all code presented between the ORG     *
;   0x004 directive and the label main can be removed. In addition    *
;   the variable assignments for 'w_temp' and 'status_temp' can       *
;   be removed.                                                       *                         
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    s65_display.asm                                   *
;    Date:          11.2005                                           *
;    File Version:  0.1                                               *
;                                                                     *
;    Author:        Mario Konegger                                    *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: This code implements a sample for steering an S65-Display *
;	    with an PIC microcontroller. The PIC Fosz is assumed to		  *
;	    be 2MHz. In cas of lower/faster speed, please adjust the 	  *
;	    wait-loop.						      						  *
;                                                                     *
;**********************************************************************


	list		p=16f876A	; list directive to define processor
	#include	<p16f876A.inc>	; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_OFF & _CPD_OFF


;***** CONSTANTS DEFINITIONS ******************************************

#define		DISPLAY_CMD	PORTB,7	; Where the lines to the CS and CMD/DATA
#define		DISPLAY_RESET	PORTB,6	; Finally the Reset-line. (Output)
#define		DISPLAY_SELECT	PORTB,5	; are connected to the PIC. (output)
;#define	__DISPLAY_ROT		; Uncomment if you want output to be rotated.
#define		CHAR_WIDTH	D'8'
#define		CHAR_HEIGHT	D'14'
#define 	DISPLAY_WIDTH 	D'132'
#define 	DISPLAY_HEIGHT	D'176'
#define		TEXT_COL	D'16'
#define		TEXT_ROW	D'12'

#define		TEXTCOLOR	0xffff
#define		BACKCOLOR	0x0000


; Define the segment for the LCD-Display routines. These include the
; temporary values to store as well as transform the numbers to decimal
; format.
lcd udata_ovr	0x2d
RES0			res	1
RES1			res	1
NUM0			res	1
NUM1			res	1
TEMPDIV			res	1
LOOPCOUNTER0	res	1
LOOPCOUNTER1	res	1
SIGNDIGIT		res	1	; Here we place the sign as well
DIGIT32			res	1	; the digits in.
DIGIT10			res	1
X_POS			res	1
Y_POS			res	1



;**********************************************************************
	CODE	0x000		; processor reset vector

	nop					; nop required for icd
  	goto    	main	; go to beginning of program


;**********************************************************************
; SPI-transmission
spi_send
	errorlevel		-302
	movwf		SSPBUF
	bsf			STATUS,RP0
	btfss		SSPSTAT,BF
	goto		$-1
	bcf			STATUS,RP0
	movf		SSPBUF,w
	return
	errorlevel		+302


spi_send_byte macro arg
	movlw		arg
	call		spi_send
	endm


spi_send_word macro arg
	movlw		high(arg)
	call		spi_send
	movlw		low(arg)
	call		spi_send
	endm


;**********************************************************************
; Initialize the S65 Display
init_display
	bsf			DISPLAY_RESET		; Release reset.
	bcf			DISPLAY_SELECT
	bsf			DISPLAY_CMD
	movlw		0xfd
	call		spi_send
	call		spi_send
	call		spi_send
	call		spi_send

; wait, then init1 (50ms min. required acc. WEB, 68ms implemented)
	movlw		D'68'
	call		wait_ms

;INIT1:  ; 24bytes
;.byte 0xEF, 0x00, 0xEE, 0x04, 0x1B, 0x04, 0xFE, 0xFE, 0xFE, 0xFE, 0xEF, 0x90, 0x4A, 0x04
;.byte 0x7F, 0x3F, 0xEE, 0x04, 0x43, 0x06 
	spi_send_word	0xef00
	spi_send_word	0xee04
	spi_send_word	0x1b04
	spi_send_word	0xfefe
	spi_send_word	0xfefe
	spi_send_word	0xef90
	spi_send_word	0x4a04
	spi_send_word	0x7f3f
	spi_send_word	0xee04
	spi_send_word	0x4306

	
; wait, then init2 (7ms+-0ms necessary according WEB - 6ms implemented in sample-code.)
	movlw		D'6'
	call		wait_ms

;INIT2:  ; 42bytes
;.byte 0xEF, 0x90,  0x09, 0x83,  0x08, 0x00,  0x0B, 0xAF,  0x0A, 0x00,  0x05, 0x00,  0x06, 0x00
;.byte 0x07, 0x00,  0xEF, 0x00,  0xEE, 0x0C,  0xEF, 0x90,  0x00, 0x80,  0xEF, 0xB0,  0x49, 0x02 
;.byte 0xEF, 0x00,  0x7F, 0x01,  0xE1, 0x81,  0xE2, 0x02,  0xE2, 0x76,  0xE1, 0x83,  0x80, 0x01
	spi_send_word	0xef90
	spi_send_word	0x0983
	spi_send_word	0x0800
	spi_send_word	0x0baf
	spi_send_word	0x0a00
	spi_send_word	0x0500
	spi_send_word	0x0600

	spi_send_word	0x0700
	spi_send_word	0xef00
	spi_send_word	0xee0c
	spi_send_word	0xef90
	spi_send_word	0x0080
	spi_send_word	0xefb0
	spi_send_word	0x4902

	spi_send_word	0xef00
	spi_send_word	0x7f01
	spi_send_word	0xe181
	spi_send_word	0xe202
	spi_send_word	0xe276
	spi_send_word	0xe183
	spi_send_word	0x8001

; write inital screen command
	spi_send_word	0xef90
	movlw		0x00
	call		spi_send
	call		spi_send
	call		clrscr_display
	return

clrscr_display
	bsf			DISPLAY_CMD
	spi_send_byte	0xef
	spi_send_byte	0x90
	spi_send_byte	0x05
	spi_send_byte	0x00
	spi_send_byte	0x06
	spi_send_byte	0x00
	spi_send_byte	0x07
	spi_send_byte	0x00
	bcf			DISPLAY_CMD
	movlw		0xb5
	movwf		LOOPCOUNTER0
	movlw		0x80
	movwf		LOOPCOUNTER1
	movlw		0xff
clrscr
	call		spi_send
	decfsz		LOOPCOUNTER1,f
	goto		clrscr
	decfsz		LOOPCOUNTER0,f
	goto		clrscr
	bcf			DISPLAY_SELECT
	return

; This function waits exactly the number of ms, as specified in 'w' register.
; Input: w ..... contains the ms to wait
; Notice: There return and call will consume four more clocks.
;		  This function needs to be adapted to the clock speed (2MHz assumed).
wait_ms
	movwf		LOOPCOUNTER1
	movlw		D'165'
	movwf		LOOPCOUNTER0
	movwf		LOOPCOUNTER0
	decfsz		LOOPCOUNTER0,f
	goto		$-1
	decfsz		LOOPCOUNTER1,f
	goto		$-6
	return


;**********************************************************************
; This is the main-program

main
	errorlevel	-302
	clrf		STATUS
	clrf		PORTB
	bsf			STATUS,RP0		; Initialize the SPI-interface
	bcf			TRISC,3			; for interfacting the display.
	bcf			TRISC,5
	bsf			TRISC,4			; Disable SDI (make corresponding PIN an output.
	clrf		TRISB			; All Port-B pins are output.
	movlw		B'11000000'
	movwf		SSPSTAT
	bcf			STATUS,RP0
	movlw		B'00100000'		; SSPEN at max. Fosz/4 speed.
	movwf		SSPCON

	call		init_display

	goto		$
	errorlevel	+302

	END						; directive 'end of program'
